@@ -33,6 +33,7 @@ class VersionTestCase(unittest.TestCase):
 
 class BasicTestCase(unittest.TestCase):
     dbtype       = db.DB_UNKNOWN  # must be set in derived class
+    cachesize    = (0, 1024*1024, 1)
     dbopenflags  = 0
     dbsetflags   = 0
     dbmode       = 0o660
@@ -72,6 +73,15 @@ class BasicTestCase(unittest.TestCase):
 
         # create and open the DB
         self.d = db.DB(self.env)
+        if not self.useEnv :
+            if db.version() >= (4, 2) :
+                self.d.set_cachesize(*self.cachesize)
+                cachesize = self.d.get_cachesize()
+                self.assertEqual(cachesize[0], self.cachesize[0])
+                self.assertEqual(cachesize[2], self.cachesize[2])
+                # Berkeley DB expands the cache 25% accounting overhead,
+                # if the cache is small.
+                self.assertEqual(125, int(100.0*cachesize[1]/self.cachesize[1]))
         self.d.set_flags(self.dbsetflags)
         if self.dbname:
             self.d.open(self.filename, self.dbname, self.dbtype,
@@ -82,6 +92,10 @@ class BasicTestCase(unittest.TestCase):
                         dbtype = self.dbtype,
                         flags = self.dbopenflags|db.DB_CREATE)
 
+        if not self.useEnv:
+            self.assertRaises(db.DBInvalidArgError,
+                    self.d.set_cachesize, *self.cachesize)
+
         self.populateDB()
 
 
@@ -670,21 +684,20 @@ class BasicWithEnvTestCase(BasicTestCase):
 
     #----------------------------------------
 
-    if db.version() >= (4,1):
-        def test09_EnvRemoveAndRename(self):
-            if not self.env:
-                return
+    def test09_EnvRemoveAndRename(self):
+        if not self.env:
+            return
 
-            if verbose:
-                print('\n', '-=' * 30)
-                print("Running %s.test09_EnvRemoveAndRename..." % self.__class__.__name__)
+        if verbose:
+            print('\n', '-=' * 30)
+            print("Running %s.test09_EnvRemoveAndRename..." % self.__class__.__name__)
 
-            # can't rename or remove an open DB
-            self.d.close()
+        # can't rename or remove an open DB
+        self.d.close()
 
-            newname = self.filename + '.renamed'
-            self.env.dbrename(self.filename, None, newname)
-            self.env.dbremove(newname)
+        newname = self.filename + '.renamed'
+        self.env.dbrename(self.filename, None, newname)
+        self.env.dbremove(newname)
 
     #----------------------------------------
 
@@ -852,12 +865,11 @@ class BasicTransactionTestCase(BasicTestCase):
     #----------------------------------------
 
 
-        if db.version() >= (4, 0):
-            def test_txn_set_timeout(self) :
-                txn=self.env.txn_begin()
-                txn.set_timeout(1234567, db.DB_SET_LOCK_TIMEOUT)
-                txn.set_timeout(2345678, flags=db.DB_SET_TXN_TIMEOUT)
-                txn.abort()
+        def test_txn_set_timeout(self) :
+            txn=self.env.txn_begin()
+            txn.set_timeout(1234567, db.DB_SET_LOCK_TIMEOUT)
+            txn.set_timeout(2345678, flags=db.DB_SET_TXN_TIMEOUT)
+            txn.abort()
 
     #----------------------------------------
 
@@ -1136,11 +1148,12 @@ class CrashAndBurn(unittest.TestCase) :
     #    # See http://bugs.python.org/issue3307
     #    self.assertRaises(db.DBInvalidArgError, db.DB, None, 65535)
 
-    def test02_DBEnv_dealloc(self):
-        # http://bugs.python.org/issue3885
-        import gc
-        self.assertRaises(db.DBInvalidArgError, db.DBEnv, ~db.DB_RPCCLIENT)
-        gc.collect()
+    if db.version() < (4, 8) :
+        def test02_DBEnv_dealloc(self):
+            # http://bugs.python.org/issue3885
+            import gc
+            self.assertRaises(db.DBInvalidArgError, db.DBEnv, ~db.DB_RPCCLIENT)
+            gc.collect()
 
 
 #----------------------------------------------------------------------
